/*

This function Writes to a Page of a Block. Once the Page Write is completed
Flash status register is read to know if the Page Write operation is
successful or not. If the Page Write Operation fails, then the Block is 
discarded by updating the BadBlock Table.

Parameter to be passed: Page No, BlockNo, Pointer where data to be written is stored
Returns: None

*/

#include "Extern_Declaration.h"

extern short n; // Used in ISR
extern short Write_ECC_Computed_Flag; 
extern unsigned short Temp_Write_ECC[];
extern short Indicate_Table_Exist;

/**********************************************************************/
	
void PageWrite(unsigned int Wpage, unsigned int Wblock, unsigned char *Write_Buf)
{
	
	//Check if the BlockNo is good or bad.
	
	if(BadBlockTable[Wblock] == 0xBD)
	{
		printf(" %d is a Bad Block...STOP \n", Wblock);
		while(1);
	}

	
	// check if write buffer is empty or not:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	*pNFC_CMD =  PageWriteCmd[0];
		 
	/*  
	
	Set up the address cycles depending on Page and Block number passed:
	
	Column: A0-A11
	Page: 	A12-A17
	Block: 	A18-A28
	
	
	1st Address Cycle: A0-A7
	2nd Address Cycle: A8-A11
	3rd Address Cycle: A12-A19
	4th Address Cycle: A20-A27
	5th Address Cycle: A28
	
	*/
	
	short temp_Page = Wpage;
	short temp_Block = Wblock;
	
	//Column
	*pNFC_ADDR = 0;
	*pNFC_ADDR = 0;
	
	//Page
	
	temp_Page = Wblock & 3;		// Get bits A18 and A19 from Block address
	temp_Page = temp_Page << 6;	// Shift to append with Page Address
	
	Wpage = Wpage | temp_Page;
	
	// First 2 bits of Block and 6 bits of Page are sent
	*pNFC_ADDR = Wpage;	
	
	// check if write buffer is empty or not:
	
	while(!(*pNFC_STAT & WB_EMPTY));	
	
	// Next 8 bits of Block address. i.e till 1023. 
			
	Wblock = temp_Block;
	Wblock = Wblock & 0x3FC;	// Mask to get A20-A27	
	Wblock = Wblock >> 2;		// Because A18 and A19 are already sent out
	
	*pNFC_ADDR = Wblock;
	
		
	// Block address greater than 1023
	
	Wblock = temp_Block;
	Wblock = Wblock & 0xFC00;	// Mask to get A28-A30
	Wblock = Wblock >> 10;		// Because A18-A27 are already sent out
	
	*pNFC_ADDR = Wblock;
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
		
	//Configure the DMA. 
		
	*pDMA2_PERIPHERAL_MAP = 0x2000;
	*pDMA2_CONFIG = WDSIZE_16;
	*pDMA2_START_ADDR = Write_Buf;
	*pDMA2_X_COUNT = NFC_PAGE_SIZE/2;  
	*pDMA2_X_MODIFY = 2;				
	*pDMA2_Y_COUNT = 0;
	*pDMA2_Y_MODIFY = 0;
	
	*pDMA2_CONFIG |= DMAEN ;
	

	for(i = 0; i<(Flash_Page_Size/NFC_PAGE_SIZE); i++)
	{
		
		// Issue Page Write  
	
		*pNFC_RST = ECC_RST;
		*pNFC_PGCTL = PG_WR_START;
		
		// Wait for ECC to be computed.
		while(!(Write_ECC_Computed_Flag));
		Write_ECC_Computed_Flag = 0;	
	
		// Disable the DMA
		*pDMA2_CONFIG &= 0xFFFE;						
		
		// Re-config start address for next DMA transfer 
		*pDMA2_START_ADDR = *pDMA2_CURR_ADDR;
			
		// Re-enable the DMA
		*pDMA2_CONFIG |= 0x0001;
		
	}
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	/* 
	
	Now Write the ECC Values in the SPARE Area.
	Flash is a 8 bit device. So write lower byte of ECC first 
	and then higher byte into spare area.
		 
	*/
	
	//loop count: 22 bit ECC generated for every 256 which is 4 bytes/256 page		
		
	short temp_wr;
	
	for(i=0; i<(2*2*Flash_Page_Size/256); i++)
	{
		
		temp_wr = Temp_Write_ECC[i];
		//write lower byte
		*pNFC_DATA_WR = temp_wr;
		//shift to write next byte
		temp_wr >>= 8;
		*pNFC_DATA_WR = temp_wr; 
		
		while(!(*pNFC_STAT & WB_EMPTY));
		
	}
		
	
	// Issue Page Confirm code
	
	*pNFC_CMD =  PageWriteCmd[1];
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	// Clear the variables used in other functions
	
	n = 0;
	
	//Wait till NBUSY Flag sets.
	
	while(!(NBUSY_Flag));
	NBUSY_Flag = 0;

	/* 
	Read Flash Status Register to know if Page Write operation failed.
	If so, update the BadBlock Table in the Flash
	*/
	
	
	Read_Flash_Status_Register();
	
	if((Flash_Status & 0x01) != 0)
	{
		// First update the copy in the internal memory. 
		BadBlockTable[Wblock] = 0xBD;
		
		// Now write this table into Flash. Where to write is given 
		BlockErase(ResBlockNo);
		PageWrite(0, ResBlockNo, &BadBlockTable[0]);
		
	}
	
	else if(Flag1 == 1)
	{
		printf("Page Write Completed \n");
	}
	

}	
	
